<?php
/*
* BillRepository.php - Repository file
*
* This file is part of the Bill component.
*-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Bill\Repositories;

use App\Yantrana\Base\BaseRepository;
 
use App\Yantrana\Components\Bill\Models\BillModel;
use App\Yantrana\Components\Bill\Interfaces\BillRepositoryInterface;
use App\Yantrana\Components\Bill\Models\BillItemsModel;
use DB;
use App\Yantrana\Components\Package\Models\UserPackageModel;

class BillRepository extends BaseRepository
                          implements BillRepositoryInterface 
{ 
    
    /**
      * Fetch the record of Bill
      *
      * @param    int || string $idOrUid
      *
      * @return    eloquent collection object
      *---------------------------------------------------------------- */

    public function fetch($idOrUid)
    {   
        if (is_numeric($idOrUid)) {

            return BillModel::where('_id', $idOrUid)->first();
        }

        return BillModel::where('_uid', $idOrUid)->first();
    }


    /**
    * Fetch the record of Bill
    *
    * @param  int || string $idOrUid
    *
    * @return eloquent collection object
    *---------------------------------------------------------------- */
    public function fetchAllBills()
    {   
        return BillModel::select('_id', '_uid')->get();
    }

    /**
    * Fetch the record of Bill
    *
    * @param  int || string $idOrUid
    *
    * @return eloquent collection object
    *---------------------------------------------------------------- */
    public function overDueBillCounts()
    {   
        return BillModel::whereIn('status', [1, 2])->where('due_date', '<', now()->format('Y-m-d'))->count();
    }

    /**
    * Fetch user bills 
    *
    * @param  int $memberId
    *
    * @return eloquent collection object
    *---------------------------------------------------------------- */
    public function fetchUserBills($memberId, $status = [1,2,3,4])
    {   
        return BillModel::select('_id', '_uid', 'amount', 'status', 'customer_user_authorities__id')
                        ->where('member_id', '=', $memberId)
                        ->with('transactions')
        				->whereIn('status', $status)
        				->get();
    }

    /**
    * Fetch user bills 
    *
    * @param  int $userAuthorityId
    *
    * @return eloquent collection object
    *---------------------------------------------------------------- */
    public function fetchLastBills()
    {   
        return BillModel::select('__data')->get();
    }

    /**
    * Fetch bill datatable source
    * @param integer $status 
    * @return  mixed
    *---------------------------------------------------------------- */
	public function fetchBillDataTableSource($status, $lastBilsUids = [])
	{
    	$dataTableConfig = [
        	'searchable' => [
                'package_amount',
                'package_title' ,
                'name',
                'formated_due_date',
                'formated_frequency'        
            ],
            'fieldAlias' => [
                'name' => DB::raw('CONCAT(first_name, " ", last_name)'),
                'package_title'  => 'packages.title',
                'formated_amount' => 'user_packages.price',
                'formated_due_date' => 'bills.due_date',
                'formated_frequency' => 'user_packages.frequency',
                'bill_uid' => 'bills._uid'
            ],
    	];

        $query = BillModel::join('user_authorities', 'user_authorities._id', 'bills.customer_user_authorities__id')
            ->join('users', 'users._id', 'user_authorities.users__id')
            ->join('bill_items', function($b) {
                $b->on('bill_items.bills__id', '=', 'bills._id')
                  ->join('user_packages', 'user_packages._id', '=', 'bill_items.user_packages__id')
                  ->join('packages', 'packages._id', '=', 'user_packages.packages__id');
            })
            ->when(!__isEmpty($lastBilsUids), function($q) use($lastBilsUids) {
                $q->whereNotIn('bills._uid', $lastBilsUids);
            })
            ->where('due_date', '>',  now()->addDays(getConfigurationSettings('generate_bill_before_day'))->format('Y-m-d'));

        return $query->select(
            __nestedKeyValues([
                'users' => ['_id as user_id', DB::raw('CONCAT(first_name, " ", last_name) AS name')],
                'bills' => [
                    '_id', '_uid', 'amount', 'customer_user_authorities__id',  'due_date', 'bill_period_from', 'bill_period_to', 'user_authorities__id'
                ],
                'user_packages' => ['_uid as user_package_uid', 'frequency', 'price'],
                'packages' => ['title as package_title']
            ])
        )->dataTables($dataTableConfig)
        ->toArray();
    }


    /**
    * Fetch bill datatable source
    * @param integer $status 
    * @return  mixed
    *---------------------------------------------------------------- */
	public function fetchUpcommingBills($lastBillsUids)
	{
        $query = BillModel::join('user_authorities', 'user_authorities._id', 'bills.customer_user_authorities__id')
            ->join('users', 'users._id', 'user_authorities.users__id')
            ->join('bill_items', function($b) {
                $b->on('bill_items.bills__id', '=', 'bills._id')
                  ->join('user_packages', 'user_packages._id', '=', 'bill_items.user_packages__id')
                  ->join('packages', 'packages._id', '=', 'user_packages.packages__id');
            })
            ->whereBetween('due_date', [
                now()->format('Y-m-d'),
                now()->addDays(getConfigurationSettings('generate_bill_before_day'))->format('Y-m-d')
            ])
            ->where(function($q) {
                $q->whereNull('user_packages.expiry_at')
                  ->orWhere(DB::raw('DATE(user_packages.expiry_at)'), '>', now()->format('Y-m-d'));
            })
            ->when(!__isEmpty($lastBillsUids), function($q) use($lastBillsUids) {
                $q->whereNotIn('bills._uid', $lastBillsUids);
            });

        return $query->select(
            __nestedKeyValues([
                'users' => ['_id as user_id', DB::raw('CONCAT(first_name, " ", last_name) AS name')],
                'bills' => [
                    '_id', '_uid', '_uid as bill_uid', 'created_at', 'amount', 'due_date', 'discount_amount', 'tax_amount', 'status', 'customer_user_authorities__id',
                    'bill_period_from', 'bill_period_to'
                ],
                'user_packages' => ['_id as user_package_id', '_uid as user_package_uid', 'frequency', 'price'],
                'packages' => ['title as package_title', 'short_description as package_short_description']
            ])
        )->get();
    }

    /**
    * Fetch bill datatable source
    * @param integer $status 
    * @return  mixed
    *---------------------------------------------------------------- */
	public function fetchPurchasedPackage($userPackageId)
	{
        return UserPackageModel::where('_id', $userPackageId)->first();
    }

    /**
    * Fetch bill datatable source
    * @param integer $status 
    * @return  mixed
    *---------------------------------------------------------------- */
	public function fetchBillsDatatable($status)
	{   
    	$dataTableConfig = [
        	'searchable' => [
                'bills._uid',
                'amount',
                'due_date',      
                'short_description'           
            ],
            'fieldAlias' => [
                'name' => DB::raw('CONCAT(first_name, " ", last_name)'),
                'generated_at'  => 'bills.created_at',
                'formated_amount' => 'bills.amount',
                'formated_due_date' => 'bills.due_date',
                'formated_discount_amount' => 'bills.discount_amount',
                'formated_tax_amount' => 'bills.tax_amount',
                'bill_uid'    => 'bills._uid',
                'bill_period' => 'bill_period_from',
                'bill_period' => 'bill_period_to'
            ],
    	];

        $query = BillModel::join('user_authorities', 'user_authorities._id', 'bills.customer_user_authorities__id')
            ->join('users', 'users._id', 'user_authorities.users__id')->
            with('userPackage');

            if ($status === 1) { // Unpaid/ Partial Paid/ Overdue
                
                $query->where(function($q) {
                        return $q->whereIn('bills.status', [1, 2]);
                    })
                    ->orWhere(function($q) {
                        return $q->whereNotIn('bills.status', [3, 4])
                          ->where('due_date', '<', now()->format('Y-m-d'));
                    }); 

            } else if($status === 3) { // Paid

                $query->where('bills.status', 3);

            } else if($status === 4) { // Cancelled
                
                $query->where('bills.status', 4);
            }

        return $query->select(
            __nestedKeyValues([
                'users' => ['_id as user_id', DB::raw('CONCAT(first_name, " ", last_name) AS name')],
                'bills' => [
                    '_id', '_uid', '_uid as bill_uid', 'created_at', 'amount', 'due_date', 'discount_amount', 'tax_amount', 'status', 'customer_user_authorities__id',
                    'bill_period_from', 'bill_period_to',
                ]
            ])
        )->dataTables($dataTableConfig)
        ->toArray();
    }
    

    /**
    * Store bill price
    *
    * @param  array $inputData
    *
    * @return  mixed
    *---------------------------------------------------------------- */

	public function storeBill($inputData)
	{   
        $keyValues = [
            'amount',
            'user_authorities__id',
            'short_description',
            'customer_user_authorities__id',
            'due_date',
            'discount_amount',
            'tax_amount',
            'status',
            'member_id',
            'client_id',
            '__data',
            'bill_period_from',
            'bill_period_to'
        ];

	    $billModelInstance = new BillModel();
	    
	    // Check if bill record generated then return positive response
	    if ($billModelInstance->assignInputsAndSave($inputData, $keyValues)) {

	        return $billModelInstance;
	    }

	    return false;
    }


    /**
    * Store bill Items
    *
    * @param  array $inputData
    *
    * @return  mixed
    *---------------------------------------------------------------- */

	public function insert($data)
	{   
	    $billItemsModel = new BillItemsModel;

	    // Check if bill record generated then return positive response
	    if ($billItemsModel->prepareAndInsert($data)) {

	        return true;
	    }

	    return false;
    }


    /**
    * update $model
    *  
    * @param  object $model
    * @param  array $data
    *
    * @return  mixed
    *---------------------------------------------------------------- */

	public function update($model, $data)
	{   
	    // Check if $model record update then return positive response
	    if ($model->modelUpdate($data)) {

	        return true;
	    }

	    return false;
    }

    /**
    * update $model
    *  
    * @param  object $model
    * @param  array $data
    *
    * @return  mixed
    *---------------------------------------------------------------- */

	public function bunchUpdateBillItems($data)
	{   
	    // Check if $model record update then return positive response
	    if (BillItemsModel::bunchUpdate($data, '_id')) {

	        return true;
	    }

	    return false;
    }

    /**
    * update $model
    *  
    * @param  object $model
    * @param  array $data
    *
    * @return  mixed
    *---------------------------------------------------------------- */

	public function delete($model)
	{   
	    // Check if $model record update then return positive response
	    if ($model->delete()) {

	        return true;
	    }

	    return false;
    }

    /**
    * Fetch the record of Bill
    *
    * @param  int || string $idOrUid
    *
    * @return eloquent collection object
    *---------------------------------------------------------------- */
    public function preparePrintBillData($billUid)
    {
        return BillModel::leftJoin('user_authorities', 'bills.customer_user_authorities__id', '=', 'user_authorities._id')
					->leftJoin('users', 'user_authorities.users__id', '=', 'users._id')
					->leftJoin('user_profiles', 'users._id', '=', 'user_profiles.users__id')
					->leftJoin('countries', 'user_profiles.countries__id', '=', 'countries._id')
					->where('bills._uid', '=', $billUid)
					->with('billItems')
					->select(__nestedKeyValues([
						'bills.*',
						'user_authorities' => [
							'users__id',
						],
						'users' => [ DB::raw('CONCAT(first_name, " ", last_name) AS customer_name'), 'mobile_number' ],
						'user_profiles' => [
							'address_line_1',
							'address_line_2',
						],
						'countries' => [
							'name AS country_name'
						]
					]))
					->first();
    }

    /**
    * Fetch the bill details with transactions, package, package items, customer
    *
    * @param  int || string $idOrUid
    *
    * @return eloquent collection object
    *---------------------------------------------------------------- */

    public function fetchDetails($idOrUid)
    {   
        return BillModel::with(['billItems', 'transactions'])
            ->where('_uid', $idOrUid)
            ->first();
    }


    /**
    * Fetch the bill details with transactions, package, package items, customer
    *
    * @param  int || string $idOrUid
    *
    * @return eloquent collection object
    *---------------------------------------------------------------- */

    public function fetchRecentPackageBill($userPackageId)
    {   
        return BillModel::join('bill_items', 'bill_items.bills__id', '=', 'bills._id')
                    ->orderBy('due_date','desc')
                    ->where('bill_items.user_packages__id', $userPackageId)
                    ->select(__nestedKeyValues([
                        'bills.*'
                    ]))
                    ->first();
    }

    /**
	  * delete prices
	  *
	  * @param  array $inputData
	  *
	  * @return  mixed
	  *---------------------------------------------------------------- */

	public function deleteBillItems($billItemIds)
	{   
	    return BillItemsModel::whereIn('bill_items._id', $billItemIds)->delete();
    }
    
    /**
    * Insert Bills With Inte
    *
    * @param  array $inputData
    *
    * @return  mixed
    *---------------------------------------------------------------- */

	public function insertBillWithItems($data)
	{   
        foreach($data as $d) 
        {
            $billModel = new BillModel;

            if (!$bill = $this->storeBill($d)) {

                return false;
            }

            activityLog(9, $bill->_id, 11);

            if (!__isEmpty($d['items'])) {

                $billItemsModel = new BillItemsModel;
                
                // Check if bill record generated then return positive response
                if (!$billItemsModel->assignInputsAndSave($d['items'], [
                    'bills__id' => $bill->_id,
                    'short_description',
                    'amount',
                    'user_packages__id'
                ])) {
        
                    return false;
                }
            }

            activityLog(10, $billItemsModel->_id, 11);
        }

        return true;
	}
}