<?php

/*
* DashboardEngine.php - Main component file
*
* This file is part of the Dashboard component.
*-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Dashboard;

use App\Yantrana\Base\BaseEngine;

use App\Yantrana\Components\Dashboard\Repositories\DashboardRepository;
use App\Yantrana\Components\Dashboard\Blueprints\DashboardEngineBlueprint;
use App\Yantrana\Components\User\Repositories\UserRepository;
use App\Yantrana\Components\Item\Repositories\ItemRepository;
use App\Yantrana\Components\Package\Repositories\PackageRepository;
use App\Yantrana\Components\Bill\Repositories\BillRepository;
use App\Yantrana\Components\Transaction\Repositories\TransactionRepository;
use Carbon\Carbon;
use Auth;

class DashboardEngine extends BaseEngine implements DashboardEngineBlueprint
{
    /**
     * @var DashboardRepository $dashboardRepository - Dashboard Repository
     */
    protected $dashboardRepository;

    /**
     * @var UserRepository $userRepository - User Repository
     */
    protected $userRepository;

    /**
     * @var ItemRepository $itemRepository - Item Repository
     */
    protected $itemRepository;

    /**
     * @var PackageRepository $packageRepository - Package Repository
     */
    protected $packageRepository;

    /**
     * @var BillRepository $billRepository - Bill Repository
     */
    protected $billRepository;

    /**
     * Constructor
     *
     * @param DashboardRepository $dashboardRepository - Dashboard Repository
     *
     * @return void
     *-----------------------------------------------------------------------*/

    public function __construct(
        DashboardRepository $dashboardRepository,
        UserRepository $userRepository,
        ItemRepository $itemRepository,
        PackageRepository $packageRepository,
        BillRepository $billRepository,
        TransactionRepository $transactionRepository
    )
    {
        $this->dashboardRepository = $dashboardRepository;
        $this->userRepository = $userRepository;
        $this->itemRepository = $itemRepository;
        $this->packageRepository = $packageRepository;
        $this->billRepository = $billRepository;
        $this->transactionRepository = $transactionRepository;
    }

    /**
     *
     */
    private function prepareMyPackages($userAuthorityId)
    {
        $packages = $this->packageRepository->getMyPackage($userAuthorityId);
        $myPackages = [];

        if (!__isEmpty($packages)) {

            $frequencies = configItem('packages.frequencies');

            foreach ($packages as $key => $package) {

                $packageItems = [];
                //check if empty
                if (!__isEmpty($package->userAssignedSlot)) {
                    foreach ($package->userAssignedSlot as $key => $slot) {

                        if (isset($slot->itemWithSlot) && isset($slot->itemWithSlot->itemSlot->slot_time)) {
                            $fromtime = formatDateTime($slot->itemWithSlot->itemSlot->slot_time, 'g:i A');
                            $endtime = addMinutesToTime($slot->itemWithSlot->itemSlot->slot_time, $slot->itemWithSlot->itemSlot->slot_duration_minutes);
                        }

                        $packageItems[] = [
                            'package_item_title' => $slot->itemWithSlot->title,
                            'package_item_slot' => $fromtime . ' - ' . $endtime,
                        ];
                    }
                }

                $itemProductData = [];
                if (!__isEmpty($package->packages)) {
                    foreach ($package->packages as $packageValue) {
                        if (!__isEmpty($packageValue->packageItems)) {
                            foreach ($packageValue->packageItems as $packageItem) {
                                if ($packageItem->item->type == 2) { // Product Item
                                    $itemProductData[] = [
                                        'title' => $packageItem->item->title
                                    ];
                                }
                            }
                        }
                    }
                }

                $myPackages[] = [
                    'package_title' => $package->title,
                    'package_price' => moneyFormat($package->price, true, false),
                    'package_started_on' => formatDateTime($package->starts_at),
                    'package_expiry' => isset($package->expiry_at) ? formatDateTime($package->expiry_at) : null,
                    'package_frequency' => configItemString($frequencies, $package->frequency),
                    'package_items' => $packageItems,
                    'item_products' => $itemProductData
                ];
            }
        }

        return $myPackages;
    }

    /**
     * Get active data.
     *
     * @return array
     *---------------------------------------------------------------- */

    public function prepareDashboardSupportData()
    {
        $customers = $this->userRepository->fetchCustomers();

        $items = $this->itemRepository->fetchItems();

        $transactions = $this->transactionRepository->fetchTransactionOfThisMonth();

        $creditAmount = $transactions->where('status', 2)->where('type', 1)->sum('amount');

        $dabitAmount = $transactions->where('status', 2)->where('type', 2)->sum('amount');

//        $currentUser = $this->userRepository->fetchUser(Auth::id());

        $myPackages = [];
//        if ($currentUser->user_roles__id == 3 or $currentUser->user_roles__id == 2) {
//            $myPackages = $this->prepareMyPackages($currentUser->user_authority_id);
//        }

        return $this->engineReaction(1, [
            'customers' => [
                'total_count' => 0,//,
                'active_count' => 0,//$customers->where('status', 1)->count()
            ],
            'items' => [
                'total_count' => $items->count(),
                'active_count' => $items->where('status', 1)->count()
            ],
            'packages' => [
                'total_count' => $this->packageRepository->fetchPackagesCount(),
                'total_purchased_count' => $this->packageRepository->fetchUserPurchasedCount()
            ],
            'transactions' => [
                'credit' => moneyFormat($creditAmount, true, true),
                'dabit' => moneyFormat($dabitAmount, true, true),
                'total_records' => $transactions->count()
            ],
            'myPackages' => $myPackages,
            'overDueBillCount' => $this->billRepository->overDueBillCounts()
        ]);
    }
}
 