<?php
/*
* InstallationVerification.php - Controller file
*-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Installation;

use DB;
use Exception;
use Schema;
use Artisan;
use App\Yantrana\Support\Utils;

class InstallationVerification
{
    protected $projectName = 'Dietbox Fitness';
    protected $problems = [];
    protected $defaultKey = 'base64:ZeeEOXG3/jMoO/C6m3e8fl2GkndivS7pAsQUMupqxuw=';
    protected $requirements = [
        'php_version' => '7.1.3',
        'mysql_version' => '5.0.0',
        'user_table' => 'users',
        'user_find' => [
            '_id' => 1
        ],
        'sql_file' => 'member_manager.sql',
        'tables' => [
            'activity_logs' => '*',
            'bill_items' => '*',
            'bills' => '*',
            'configurations' => '*',
            'countries' => '*',
            'email_change_requests' => '*',
            'items' => '*',
            'login_attempts' => '*',
            'login_logs' => '*',
            'package_items' => '*',
            'packages' => '*',
            'password_resets' => '*',
            'payment_transactions' => '*',
            'prices' => '*',
            'slots' => '*',
            'token_registry' => '*',
            'user_authorities' => '*',
            'user_packages' => '*',
            'user_profiles' => '*',
            'user_roles' => '*',
            'user_slots' => '*',
            'users' => '*',
        ]
    ];

    protected $styleLines = "padding:20px; border-bottom:1px dotted #ddd;margin:16px;";

    /**
     * Constructor.
     *
     *-----------------------------------------------------------------------*/
    public function __construct()
    {
    }

    /**
     * Verify the installation
     *
     * @return void
     *---------------------------------------------------------------- */
    public function verify()
    {
        if (! file_exists('verify-install.php')) {
            abort(404);
        }

        echo '<!DOCTYPE html><html lang="en"><head><meta charset="UTF-8"><title>'.$this->projectName.' Installation Verification</title>

                <style>
                    .lw-button {
                    background-color: #64c11f; /* Green */
                    border: none;
                    color: white;
                    padding: 8px 10px;
                    text-align: center;
                    text-decoration: none;
                    display: inline-block;
                    font-size: 16px;
                    margin: 4px 2px;
                    cursor: pointer;
                    border-radius: 5px;
                }
                </style>

                </head><body style="text-align:center;">
            ';

        echo "<h2>".$this->projectName." Installation Verification!!</h2>";

        if (env('APP_KEY') === $this->defaultKey) {
            Artisan::call('key:generate');
             echo "<div style='$this->styleLines'>✓ New <strong style='color:green'>API KEY</strong> Generated</div>";   
            $this->updateEnvValue('APP_ENV', 'production');
            echo "<div style='$this->styleLines'>✓ New <strong style='color:green'>APP_ENV</strong> set to production</div>";   
            $generatedKeys = $this->generateFormEncryptionKeys();
            // store password
            $this->updateTechConfigItem('default_rsa_passphrase', $generatedKeys['password']);
            // store public key
            $this->updateTechConfigItem('default_rsa_public_key', $generatedKeys['publicKey']);
            // store private key
            $this->updateTechConfigItem('default_rsa_private_key', $generatedKeys['privateKey']);
            echo "<div style='$this->styleLines'>✓ New <strong style='color:green'>Form Encryption Keys</strong> Generated</div>";
        } else {
            echo "<div style='$this->styleLines'>✓ <strong style='color:green'>API KEY</strong> exists</div>";
        }
            

        try {
            $dbPdo = DB::connection()->getPdo();
            echo "<div style='$this->styleLines'>✓ <strong style='color:green'>Database</strong> Connected</div>";

            try {
                $missingTables = [];

                foreach ($this->requirements['tables'] as $key => $value) {
                    if (! Schema::hasTable($key)) {
                        $missingTables[] = $key;
                    }
                }

                if (! empty($missingTables)) {
                    echo "<div style='$this->styleLines'>✗ <span style='color:red'>MySQL Queries</span> needs to be executed from <span style='color:red'>".$this->requirements['sql_file']."</span>. <br> <strong>".implode(', ', $missingTables)."</strong> table(s) are missing. Read setup guide. </div>";

                    $this->problems[] = 'missing-tables';
                } else {
                    echo "<div style='$this->styleLines'>✓ <strong style='color:green'>MySQL Queries</strong> seems to be executed. Required Tables are exists.</div>";

                    $adminUser = DB::table($this->requirements['user_table'])->where($this->requirements['user_find'])->first();

                    if (__isEmpty($adminUser)) {
                        echo "<div style='$this->styleLines'>✗ <span style='color:red'>Admin user</span> is missing. You may not executed all the queries from <span style='color:red'>".$this->requirements['sql_file']."</span>, Cross check that you have executed all the SQL insert queries properly.</div>";

                        $this->problems[] = 'missing-admin-user';
                    } else {
                        echo "<div style='$this->styleLines'>✓ <strong style='color:green'>Admin user</strong> is present.</div>";
                    }
                }

                /* } else {
                      echo "<div style='$this->styleLines color:red'>✗ Invalid MySQL Version it should be greater than ".$this->requirements['mysql_version']." </div>";

                     $this->problems[] = 'mysql_version';
                 }*/
            } catch (Exception $e) {
                echo "<div style='$this->styleLines color:red'>".$e->getMessage()."</div>";
                // echo "<div style='$this->styleLines color:red'>✗ Unable to connect to database. Please correct your database settings in <strong>.env.php</strong>. Follow the setup guide.</div>";

                $this->problems[] = 'database';
            }
        } catch (Exception $e) {
            echo "<div style='$this->styleLines color:red'>✗ Unable to connect to database. Please correct your database settings in <strong>.env.php</strong>. Follow the setup guide.</div>";

            $this->problems[] = 'database';
        }

        if (empty($this->problems)) {
            echo "<div style='text-align:center;'><h1 style='color:green'>Congratulation!! All seems to be GOOD!!<br></h1>You can proceed, Please delete <strong style='color:red;'>verify-install.php</strong></div>";
            echo "<br><br><a class='lw-button' href='".route('manage.app')."'>Go to login</a>";
        } else {
            echo "<div style='text-align:center;'><h1 style='color:red'>Ooooops .... Please fix highlighted issues in RED above.</h1></div>";
        }

        echo '</body></html>';
    }

    /**
     * Update __tech config item file
     *
     * @return void
     *---------------------------------------------------------------- */
    protected function updateTechConfigItem($itemName, $itemValue)
    {
        return file_put_contents(config_path('__tech.php'), str_replace(
            config('__tech.form_encryption.'.$itemName),
            $itemValue,
            file_get_contents(config_path('__tech.php'))
        ));
    }

    /**
     * Generate Keys for Form Encryption
     *
     * @return void
     *---------------------------------------------------------------- */
    protected function generateFormEncryptionKeys()
    {
        $rsa = new \phpseclib\Crypt\RSA();
        $password = Utils::generateStrongPassword(43);

        $rsa->comment = $this->projectName." YesSecurity";
        //$rsa->setPublicKeyFormat(6);
        $rsa->setPassword($password);

        $keysCreated = $rsa->createKey(2048);

        return [
            'publicKey'  => $keysCreated['publickey'],
            'privateKey' => $keysCreated['privatekey'],
            'password'  => $password
        ];
    }

    /**
     * Update ENV file values
     * src - https://stackoverflow.com/questions/40450162/how-to-set-env-values-in-laravel-programmatically-on-the-fly
     * @return void
     *---------------------------------------------------------------- */
    protected function updateEnvValue($envKey, $envValue)
    {
        $envFile = app()->environmentFilePath();
        $str = file_get_contents($envFile);

        //$oldValue = strtok($str, "{$envKey}=");
        $oldValue = env($envKey);

        $str = str_replace("{$envKey}={$oldValue}", "{$envKey}={$envValue}", $str);
        //$str = str_replace("{$envKey}={$oldValue}", "{$envKey}={$envValue}\n", $str);

        $fp = fopen($envFile, 'w');
        fwrite($fp, $str);
        fclose($fp);
    }
}
