<?php
/*
* ItemEngine.php - Main component file
*
* This file is part of the Item component.
*-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Item;

use App\Yantrana\Base\BaseEngine;
 
use App\Yantrana\Components\Item\Repositories\ItemRepository;
use App\Yantrana\Components\Package\Repositories\PackageRepository;
use App\Yantrana\Components\Slots\Repositories\SlotsRepository;
use App\Yantrana\Components\Item\Interfaces\ItemEngineInterface;


class ItemEngine extends BaseEngine implements ItemEngineInterface 
{   
     
    /**
     * @var  ItemRepository $itemRepository - Item Repository
     */
    protected $itemRepository;

    /**
     * @var  PackageRepository $packageRepository - Package Repository
     */
    protected $packageRepository;

    /**
    * @var  SlotRepository $slotRepository - Slot Repository
    */
    protected $slotRepository;
    

    /**
      * Constructor
      *
      * @param  ItemRepository $itemRepository - Item Repository
      * @param  PackageRepository $packageRepository - Package Repository
      * @param  SlotRepository $slotRepository - Slot Repository
      *
      * @return  void
      *-----------------------------------------------------------------------*/

    function __construct(
        ItemRepository $itemRepository,
        PackageRepository $packageRepository,
        SlotsRepository $slotRepository
    )
    {
        $this->itemRepository = $itemRepository;
        $this->packageRepository = $packageRepository;
        $this->slotRepository = $slotRepository;
    }
    
    
  /**
  	* Item datatable source 
  	*
  	* @return  array
  	*---------------------------------------------------------------- */
 
	public function prepareItemDataTableSource()
	{   
    	$itemCollection = $this->itemRepository->fetchItemDataTableSource();

        $config = configItem('item.type');

    	$requireColumns = [
        	'_id',   
            '_uid', 
            'title',
            'formated_created' => function($key) {
                return formatDateTime($key['created_at']);    
            },
            'type' => function($key) use($config) {
                return configItemString($config, $key['type']);    
            },
            'status' => function($key){
                return techItemString($key['status']);    
            },
            'formated_updated_at' => function($key) {
                return formatDateTime($key['updated_at']);    
            },
            'can_edit_item' => function() {
            	return canAccess('edit_item'); 
            },
            'can_delete_item'  => function() {
            	return canAccess('delete_and_restore_item');
            },
            'view_item_details'  => function() {
                return canAccess('view_item_details');
            },
        ];

    	return $this->dataTableResponse($itemCollection, $requireColumns);
	}
 

	/**
	  * Item delete process 
	  *
	  * @param  mix $itemIdOrUid
	  *
	  * @return  array
	  *---------------------------------------------------------------- */

	public function processItemDelete($itemIdOrUid)
	{   
	    $item = $this->itemRepository->fetch($itemIdOrUid);

	    if (__isEmpty($item)) {
	       return $this->engineReaction(18, null, __tr('Item not found.'));
        }

        $itemId = $item->_id;

        $packageItems = $this->packageRepository->getAllPackageItemsByItem($item->_id);
 
        if (!__isEmpty($packageItems)) {
	       return $this->engineReaction(2, null, __tr('Item cannot be deleted as it is used in Package.'));
        }

	    if ($this->itemRepository->deleteItem($item)) {

            activityLog(12, $itemId, 3); 

	        return $this->engineReaction(1, null, __tr('Item deleted.'));
	    }

	    return $this->engineReaction(2, null, __tr('Item not deleted.'));
	}
 
    /**
      * Item Add Support Data
      *
      * @return  array
      *---------------------------------------------------------------- */

    public function prepareItemSupportData()
    {   
        $config = configItem('item');

        return $this->engineReaction(1, [
            'item' => [
                'type_config' => $config['type'],
                'slot_frequency_config' => $config['slot_frequency']
            ]
        ]);
    }
	/**
	  * Item create 
	  *
	  * @param  array $inputData
	  *
	  * @return  array
	  *---------------------------------------------------------------- */

	public function processItemCreate($request)
	{
        $inputData = $request->all();
        $inputData['client_id']= $request->user()->client_id;
        $transation = $this->itemRepository->processTransaction(function() use($inputData) {

            if (!$item = $this->itemRepository->storeItem($inputData)) {

                return $this->itemRepository->transactionResponse(2, null, __tr('Item not added.'));
            }

            activityLog(12, $item->_id, 12);

            $inputedSlots = $inputData['slots'] ?? [];

            if ($inputData['type'] == 1 and !__isEmpty($inputedSlots)) {

                $slotsData = [];

                foreach ($inputedSlots as $key => $slot) {

                    $slotsData[] = [
                        'title' =>  $slot['title'],
                        'client_id' =>  $inputData['client_id'],
                        'slot_frequency' =>  $slot['slot_frequency'],
                        'slot_duration_minutes' =>  $slot['slot_duration_minutes'],
                        'slot_time' =>  $slot['slot_time'],
                        'items__id' => $item->_id,
                        'status' => 1
                    ];
                    
                }
        
                if (!$this->slotRepository->insert($slotsData)) {

                    return $this->itemRepository->transactionResponse(2, null, __tr('Slot not added.'));
                }

                activityLog(13, $item->_id, 12);
            }

            return $this->itemRepository->transactionResponse(1, null, __tr('Item added.'));

        });

        return $this->engineReaction($transation);

	}


	/**
	  * Item create 
	  *
	  * @param  array $inputData
	  *
	  * @return  array
	  *---------------------------------------------------------------- */

	public function processItemSlotCreate($request, $itemIdOrUid)
	{
        $inputData =  $request->all();
        dd($request->user(),$inputData);
	 	$item = $this->itemRepository->fetch($itemIdOrUid);
		
        // Check if $item not exist then throw not found 
    	if (__isEmpty($item)) {
         	return $this->engineReaction(18, null, __tr('Item not found.'));
        }

        $transactionResponse = $this->itemRepository->processTransaction(function() use($inputData, $item) {

            $inputedSlots = $inputData['slots'] ?? [];

            if (!__isEmpty($inputedSlots)) {

                $slotsData = [];

                foreach ($inputedSlots as $key => $slot) {

                    $slotsData[] = [
                        'title' =>  $slot['title'],
                        'client_id' =>  $slot['title'],
                        'slot_frequency' =>  $slot['slot_frequency'],
                        'slot_duration_minutes' =>  $slot['slot_duration_minutes'],
                        'slot_time' =>  $slot['slot_time'],
                        'items__id' => $item->_id,
                        'status' => 1
                    ];
                    
                }
        
                if (!$this->slotRepository->insert($slotsData)) {

                    return $this->itemRepository->transactionResponse(2, null, __tr('Slot not added.'));
                }

                activityLog(13, $item->_id, 12);
            }

            return $this->itemRepository->transactionResponse(1, null, __tr('Item Slots added.'));

        });

        return $this->engineReaction($transactionResponse);

	}
 
    /**
      * Item prepare update data 
      *
      * @param  mix $itemIdOrUid
      *
      * @return  array
      *---------------------------------------------------------------- */

	public function prepareItemUpdateData($itemIdOrUid)
	{  
    	$item = $this->itemRepository->fetch($itemIdOrUid);

        // Check if $item not exist then throw not found 
        // exception
    	if (__isEmpty($item)) {
         	return $this->engineReaction(18, null, __tr('Item not found.'));
        }

        $slotsData = [];

        if ($item->type === 1) {  // Service

            $slots = $this->slotRepository->fetchItemSlots($item->_id)->toArray();

            if (__isEmpty($slots)) {
                return $this->engineReaction(18, null, __tr('Item slots not found.'));
            }

            $count = 1;

            foreach ($slots as $key => $slot) {
 
                $slotsData[] = [
                    'rowId' => 'row'.$count,
                    'title' => $slot['title'],
                    '_uid'  => $slot['_uid'],
                    'slot_frequency' => $slot['slot_frequency'],
                    'slot_duration_minutes' => $slot['slot_duration_minutes'],
                    'slot_time' => formatTime($slot['slot_time'], 'h:i'),
                    'show_delete_warning' => !__isEmpty($slot['user_slots'])
                ];

                $count++;
            }
        
        }

        $data = [
            '_id'    => $item->_id,
            '_uid'   => $item->_uid,
            'title'  => $item->title,
            'short_description'  => $item->short_description,
            'status' => $item->status,
            'type'   => $item->type,
            'slot_type' => $item->slot_type,
            'slots'  => $slotsData,
        ];

        $config = configItem('item');

    	return $this->engineReaction(1, [
            'item' => $data,
            'config' => [
                'type_config' => $config['type'],
                'slot_frequency_config' => $config['slot_frequency']
            ]
        ]);
	}

    /**
	  * Item process update 
	  * 
	  * @param  mix $itemIdOrUid
	  * @param  array $inputData
	  *
	  * @return  array
	  *---------------------------------------------------------------- */

	public function processItemUpdate($itemIdOrUid, $request)
	{
	    $inputData =  $request->all();
        $inputData['client_id'] = $request->user()->client_id;
    	$item = $this->itemRepository->fetch($itemIdOrUid);

        // Check if $item not exist then throw not found 
        // exception
    	if (__isEmpty($item)) {
         	return $this->engineReaction(18, null, __tr('Item not found.'));
        }
        
        $transactions = $this->itemRepository->processTransaction(function($key) use($item, $inputData) {

            $updateData = [
                'title' => $inputData['title'],      
                'short_description' => $inputData['short_description'],
                'status' => $inputData['status']      
            ];
            
            $isUpdateData = false;
    
            // Check if Item updated
            if ($this->itemRepository->update($item, $updateData)) {
    
                $isUpdateData = true;
    
                activityLog(12, $item->_id, 2);
            }
            
            $inputSlots = $inputData['slots'] ?? [];
    
            if ($item->type === 1 and !__isEmpty($inputSlots)) { // service
                
                $slots = $this->slotRepository->fetchItemSlots($item->_id)->toArray();

                if (__isEmpty($slots)) {
                    return $this->itemRepository->transactionResponse(18, null, __tr('Item slots not found.'));    
                }

                $existingSlots = $newSlots = $removedSlots = $existingAndNewInputSlots = [];

                foreach($inputSlots as $inputSlot) {

                    $inputSlotUid = $inputSlot['_uid'] ?? '';

                    if (__isEmpty($inputSlotUid)) { 

                        $newSlots[] = [
                            'title' =>  $inputSlot['title'],
                            'client_id' =>  $inputData['client_id'],
                            'slot_frequency' =>  $inputSlot['slot_frequency'],
                            'slot_duration_minutes' =>  $inputSlot['slot_duration_minutes'],
                            'slot_time' =>  $inputSlot['slot_time'],
                            'items__id' => $item->_id,
                            'status' => 1
                        ];
                    }

                    if (!__isEmpty($inputSlotUid)) {
                        $existingAndNewInputSlots[$inputSlot['_uid']] = $inputSlot; 
                    }
                }

                foreach($slots as $slot) 
                {
                    if (array_key_exists($slot['_uid'], $existingAndNewInputSlots)) {

                        $findedSlot = $existingAndNewInputSlots[$slot['_uid']];

                        $existingSlots[] = [
                            'title' =>  $findedSlot['title'],
                            '_uid' =>  $slot['_uid'],
                            'slot_frequency' =>  $findedSlot['slot_frequency'],
                            'slot_duration_minutes' =>  $findedSlot['slot_duration_minutes'],
                            'slot_time' =>  $findedSlot['slot_time']
                        ];
                    }

                    if (!array_key_exists($slot['_uid'], $existingAndNewInputSlots)) {
                        $removedSlots[] = $slot['_id'];
                    }

                }

                // New Slots
                if ($newSlots) {
                    
                    if (!$this->slotRepository->insert($newSlots)) {

                        return $this->itemRepository->transactionResponse(2, null, __tr('Slot not added.'));
                    }
    
                    activityLog(13, $item->_id, 12);

                    $isUpdateData = true;
                }


                // Remove Slots
                if ($removedSlots) {

                    if (!$this->slotRepository->deleteItemSlots($removedSlots)) {

                        return $this->itemRepository->transactionResponse(2, null, __tr('Slot not removed.'));
                    }
    
                    activityLog(13, $item->_id, 8);

                    $isUpdateData = true;
                }


                // slotBunchUpdate
                if (!__isEmpty($existingSlots) and $this->slotRepository->slotBunchUpdate($existingSlots, '_uid')) {

                    activityLog(13, $item->_id, 2);

                    $isUpdateData = true;
                }

            }
    
            if ($isUpdateData) {
                return $this->itemRepository->transactionResponse(1, null, __tr('Item updated.'));
            }
    
            return $this->itemRepository->transactionResponse(14, null, __tr('Item not updated.'));

        });

        return $this->engineReaction($transactions);
        
    }
    
    /**
      * Item prepare details
      *
      * @param  mix $itemIdOrUid
      *
      * @return  array
      *---------------------------------------------------------------- */

	public function prepareItemDetails($itemIdOrUid)
	{  
    	$item = $this->itemRepository->fetch($itemIdOrUid);

        // Check if $item not exist then throw not found 
        // exception
    	if (__isEmpty($item)) {
         	return $this->engineReaction(18, null, __tr('Item not found.'));
        }

        $slotsData = [];

        if ($item->type === 1) {  // Service

            $slots = $this->slotRepository->fetchItemSlots($item->_id)->toArray();

            if (__isEmpty($slots)) {
                return $this->engineReaction(18, null, __tr('Item slots not found.'));
            }

            $count = 1;

            foreach ($slots as $key => $slot) {
 
                $slotsData[] = [
                    'title' => $slot['title'],
                    'slot_frequency' => configItemString(configItem('item.slot_frequency'), $slot['slot_frequency']),
                    'slot_duration_minutes' => ($slot['slot_duration_minutes']),
                    'slot_time' => formatTime($slot['slot_time'], 'h:i a'),
                    'show_delete_warning' => !__isEmpty($slot['user_slots'])
                ];

                $count++;
            }
        
        }

        $data = [
            'title'  => $item->title,
            'short_description' => $item->short_description,
            'status' => techItemString($item->status),
            'type'   => configItemString(configItem('item.type'), $item->type),
            'slots'  => $slotsData
        ];

    	return $this->engineReaction(1, [
            'item' => $data
        ]);
	}
}