<?php
/*
* PackageEngine.php - Main component file
*
* This file is part of the Package component.
*-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Package;

use App\Yantrana\Base\BaseEngine;

use App\Yantrana\Components\Package\Repositories\PackageRepository;
use App\Yantrana\Components\Item\Repositories\ItemRepository;
use App\Yantrana\Components\User\Repositories\UserRepository;
use App\Yantrana\Components\Slots\Repositories\SlotsRepository;
use App\Yantrana\Components\Package\Interfaces\PackageEngineInterface;
use App\Yantrana\Components\Bill\Repositories\BillRepository;
use App\Yantrana\Components\Transaction\Repositories\TransactionRepository;

use Carbon\Carbon;
use App\Yantrana\Components\User\UserTrait;

class PackageEngine extends BaseEngine implements PackageEngineInterface
{

    use UserTrait;

    /**
     * @var  PackageRepository $packageRepository - Package Repository
     */
    protected $packageRepository;

    /**
     * @var  ItemRepository $itemRepository - Item Repository
     */
    protected $itemRepository;

    /**
     * @var  UserRepository $userRepository - User Repository
     */
    protected $userRepository;

    /**
     * @var  SlotsRepository $slotsRepository - Slots Repository
     */
    protected $slotsRepository;

    /**
     * @var  BillRepository $billRepository - Bill Repository
     */
    protected $billRepository;

    /**
     * @var  TransactionRepository $transactionRepository - Transaction Repository
     */
    protected $transactionRepository;

    /**
     * Constructor
     *
     * @param PackageRepository $packageRepository - Package Repository
     * @param ItemRepository $itemRepository - Item Repository
     * @param UserRepository $userRepository - User Repository
     * @param SlotsRepository $slotsRepository - Slots Repository
     * @param BillRepository $billRepository - Bill Repository
     * @param TransactionRepository $transactionRepository - Transaction Repository
     *
     * @return  void
     *-----------------------------------------------------------------------*/

    function __construct(PackageRepository $packageRepository, ItemRepository $itemRepository, UserRepository $userRepository, SlotsRepository $slotsRepository, BillRepository $billRepository, TransactionRepository $transactionRepository)
    {
        $this->packageRepository = $packageRepository;
        $this->itemRepository = $itemRepository;
        $this->userRepository = $userRepository;
        $this->slotsRepository = $slotsRepository;
        $this->billRepository = $billRepository;
        $this->transactionRepository = $transactionRepository;
    }


    /**
     * Package datatable source
     *
     * @return  array
     *---------------------------------------------------------------- */

    public function preparePackageDataTableSource()
    {
        $packageCollection = $this->packageRepository->fetchPackageDataTableSource();

        $requireColumns = [
            '_id',
            '_uid',
            'title',
            'short_description',
            'formatted_status' => function ($key) {
                return techItemString($key['status']);
            },
            'created_at' => function ($key) {
                return formatDateTime($key['created_at']);
            },
            'updated_at' => function ($key) {
                return formatDateTime($key['updated_at']);
            },
            'canViewDetails' => function ($key) {
                return canAccess('view_package_details');
            },
            'canEdit' => function ($key) {
                return canAccess('edit_package');
            },
            'canDelete' => function ($key) {
                return canAccess('delete_and_restore_package');
            }
        ];

        return $this->dataTableResponse($packageCollection, $requireColumns);
    }


    /**
     * Package delete process
     *
     * @param mix $packageIdOrUid
     *
     * @return  array
     *---------------------------------------------------------------- */

    public function processPackageDelete($packageIdOrUid)
    {
        $package = $this->packageRepository->fetch($packageIdOrUid);

        //check if exists
        if (__isEmpty($package)) {
            return $this->engineReaction(18, null, __tr('Package not found.'));
        }

        $assignedPackages = $this->packageRepository->getAllAssignedPackages($package->_id);

        //check if package is assigned, so will not delete it
        if (!__isEmpty($assignedPackages)) {
            return $this->engineReaction(2, null, __tr('Package you are deleting has been assigned to a user, so it cannot be deleted.'));
        }

        //delete package
        if ($this->packageRepository->deletePackage($package)) {

            activityLog(7, $package->_id, 3);

            return $this->engineReaction(1, null, __tr('Package deleted.'));
        }

        return $this->engineReaction(2, null, __tr('Package not deleted.'));
    }

    /**
     * Package Add Support Data
     *
     * @return  array
     *---------------------------------------------------------------- */

    public function preparePackageSupportData()
    {
        $items = $this->itemRepository->getAllItems();

        $services = [];
        $products = [];
        if (!__isEmpty($items)) {

            foreach ($items as $key => $item) {

                if ($item->status == 1) {
                    if ($item->type == 1) {
                        $services[] = [
                            'item_id' => $item->_id,
                            'title' => $item->title,
                            'checked' => false
                        ];
                    } else if ($item->type == 2) {
                        $products[] = [
                            'item_id' => $item->_id,
                            'title' => $item->title,
                            'checked' => false
                        ];
                    }
                }
            }
        }

        return $this->engineReaction(1, [
            'packageitems' => [
                'services' => $services,
                'products' => $products,
            ],
            'package_frequencies' => configItem('packages.frequencies'),
            'currency_symbol' => getCurrencySymbol()
        ]);
    }

    /**
     * Package create
     *
     * @param array $inputData
     *
     * @return  array
     *---------------------------------------------------------------- */

    public function processPackageCreate($request)
    {
        $inputData = $request->all();
        $inputData['client_id'] = $request->user()->client_id;
        $transactionResponse = $this->packageRepository->processTransaction(function () use ($inputData) {

            $package = $this->packageRepository->storePackage($inputData);

            //check if package items selected
            if (!isset($inputData['selected_items'])) {
                return $this->packageRepository->transactionResponse(2, null, __tr('Please select package items.'));
            }
            //check if package items selected
            if (isset($inputData['selected_items']) && __isEmpty($inputData['selected_items'])) {
                return $this->packageRepository->transactionResponse(2, null, __tr('Please select package items.'));
            }

            if (!__isEmpty($inputData['selected_items'])) {
                foreach ($inputData['selected_items'] as $items) {
                    $inputData['package_items'][] = [
                        'packages__id' => $package->_id,
                        'items__id' => $items,
                        'status' => 1
                    ];
                }
            }

            //check if package frequencies selected
            if (!isset($inputData['package_frequency'])) {
                return $this->packageRepository->transactionResponse(2, null, __tr('Please select package frequency items.'));
            }

            //check if package items selected
            if (isset($inputData['package_frequency']) && __isEmpty($inputData['package_frequency'])) {
                return $this->packageRepository->transactionResponse(2, null, __tr('Please select package items frequency'));
            }

            $frequencyData = [];
            if (!__isEmpty($inputData['package_frequency'])) {
                foreach ($inputData['package_frequency'] as $key => $price) {

                    if (isset($price['checked']) && $price['checked'] == true) {
                        $frequencyData[] = [
                            'price' => $price['package_price'],
                            'frequency' => $key,
                            'packages__id' => $package->_id,
                            'status' => 1
                        ];
                    }
                }
            }

            //check if package items selected 
            if (__isEmpty($frequencyData)) {
                return $this->packageRepository->transactionResponse(2, null, __tr('Please select package items frequency'));
            }

            if ($package) {
                activityLog(7, $package->_id, 1);
                $this->packageRepository->storePackageItems($inputData['package_items'], $package->_id);

                $this->packageRepository->storeFrequencyPrices($frequencyData);


                return $this->packageRepository->transactionResponse(1, null, 'Package added.');
            }

            return $this->packageRepository->transactionResponse(2, null, 'Package not added.');
        });

        return $this->engineReaction($transactionResponse);
    }


    /**
     * Package prepare update data
     *
     * @param mix $packageIdOrUid
     *
     * @return  array
     *---------------------------------------------------------------- */

    public function preparePackageUpdateData($packageIdOrUid)
    {
        $package = $this->packageRepository->fetch($packageIdOrUid);

        if (__isEmpty($package)) {
            return $this->engineReaction(18, null, __tr('Package not found.'));
        }

        $allPackageItems = $this->packageRepository->getAllPackageItems($package->_id);
        $packageItems = $allPackageItems->pluck('items__id')->toArray();
        $items = $this->itemRepository->getAllItems();

        $services = [];
        $products = [];
        if (!__isEmpty($items)) {

            foreach ($items as $key => $item) {

                if ($item->status == 1) {
                    if ($item->type == 1) {
                        $services[] = [
                            'item_id' => $item->_id,
                            'title' => $item->title,
                            'checked' => in_array($item->_id, $packageItems)
                        ];
                    } else if ($item->type == 2) {
                        $products[] = [
                            'item_id' => $item->_id,
                            'title' => $item->title,
                            'checked' => in_array($item->_id, $packageItems)
                        ];
                    }
                }
            }
        }

        $addedFreq = [];
        if (!__isEmpty($package->packagePrices)) {
            foreach ($package->packagePrices as $key => $price) {

                $addedFreq[$price['frequency']] = [
                    'checked' => true,
                    'package_price' => $price->price,
                    'frequency' => $price->frequency,
                    'prev_added' => true,
                    '_id' => $price->_id
                ];
            }
        }

        $packageData = [
            '_id' => $package->_id,
            '_uid' => $package->_uid,
            'short_description' => $package->short_description,
            'title' => $package->title,
            'status' => $package->status,
            'package_frequency' => $addedFreq
        ];

        return $this->engineReaction(1, [
            'packageData' => $packageData,
            'packageitems' => [
                'services' => $services,
                'products' => $products
            ],
            'added_frequencies' => $addedFreq,
            'package_frequencies' => configItem('packages.frequencies'),
            'currency_symbol' => getCurrencySymbol()
        ]);
    }

    /**
     * Package process update
     *
     * @param mix $packageIdOrUid
     * @param array $inputData
     *
     * @return  array
     *---------------------------------------------------------------- */

    public function processPackageUpdate($packageIdOrUid, $inputData)
    {
        $transactionResponse = $this->packageRepository->processTransaction(function () use ($packageIdOrUid, $inputData) {

            $package = $this->packageRepository->fetch($packageIdOrUid);

            if (__isEmpty($package)) {
                return $this->packageRepository->transactionResponse(18, null, __tr('Package not found.'));
            }

            $allPackageItems = $this->packageRepository->getAllPackageItems($package->_id);
            $packageItems = $allPackageItems->pluck('items__id')->toArray();

            /*--------------------related to package items ---------------------------*/
            $insertItems = [];
            $deleteItems = [];
            $noChangeItems = [];

            $mergedItems = array_unique(array_merge($inputData['selected_items'], $packageItems));

            if (!__isEmpty($mergedItems)) {
                foreach ($mergedItems as $items) {

                    if (in_array($items, $packageItems) && in_array($items, $inputData['selected_items'])) {
                        $noChangeItems[] = $items;
                    } else if (in_array($items, $packageItems) && !in_array($items, $inputData['selected_items'])) {
                        $deleteItems[] = $items;
                    } else if (!in_array($items, $packageItems) && in_array($items, $inputData['selected_items'])) {
                        $insertItems[] = [
                            'packages__id' => $package->_id,
                            'items__id' => $items,
                            'status' => 1
                        ];
                    }
                }
            }

            $newStoredPackageItems = false;
            $deletedPackageItems = false;
            $updatedPackagePrice = false;

            //check if items inserted then store package items
            if (!__isEmpty($insertItems)) {
                $newStoredPackageItems = $this->packageRepository->storePackageItems($insertItems, $package->_id);
            }

            //check if items inserted then store package items
            if (!__isEmpty($deleteItems)) {
                $deletedPackageItems = $this->packageRepository->deletePackageItems($deleteItems, $package->_id);
            }
            /*--------------------related to package items ---------------------------*/

            /*--------------------------prices related----------------------------*/
            $newFrequencyData = [];
            $updateFrequencyData = [];
            $deleteFrequencyData = [];

            if (!__isEmpty($inputData['package_frequency'])) {

                foreach ($inputData['package_frequency'] as $key => $freq) {

                    if (isset($freq['checked']) && $freq['checked'] == true) {

                        if (!isset($freq['prev_added'])) {
                            $newFrequencyData[] = [
                                'price' => $freq['package_price'],
                                'frequency' => $key,
                                'packages__id' => $package->_id,
                                'status' => 1
                            ];
                        } else if (isset($freq['prev_added']) && isset($freq['_id'])) {
                            $updateFrequencyData[] = [
                                'price' => $freq['package_price'],
                                'frequency' => $key,
                                'packages__id' => $package->_id,
                                'status' => 1,
                                '_id' => $freq['_id']
                            ];
                        }
                    } else {
                        if (isset($freq['_id'])) {
                            $deleteFrequencyData[] = $freq['_id'];
                        }
                    }
                }
            }

            $packageCount = count($inputData['package_frequency']);

            //check if package frequencies selected
            if ($packageCount == count($deleteFrequencyData)) {
                return $this->packageRepository->transactionResponse(2, null, __tr('Please select package frequency items.'));
            }

            //check if package items selected
            if (isset($inputData['selected_items']) && __isEmpty($inputData['selected_items'])) {
                return $this->packageRepository->transactionResponse(2, null, __tr('Please select package items.'));
            }


            $newfreq = false;
            $updatedfreq = false;
            $deletedfreq = false;
            if (!__isEmpty($newFrequencyData)) {
                $newfreq = $this->packageRepository->storeFrequencyPrices($newFrequencyData);
            }

            if (!__isEmpty($updateFrequencyData)) {
                $updatedfreq = $this->packageRepository->updateFrequencyPrices($updateFrequencyData);
            }

            if (!__isEmpty($deleteFrequencyData)) {
                $deletedfreq = $this->packageRepository->deleteFrequencyPrices($deleteFrequencyData, $package->_id);
            }
            /*--------------------------prices related----------------------------*/

            $updateData = [
                'title' => $inputData['title'],
                'short_description' => $inputData['short_description'] ?? [],
                'status' => $inputData['status']
            ];

            if ($this->packageRepository->update($package, $updateData) || $newStoredPackageItems || $deletedPackageItems || $updatedPackagePrice || $newfreq || $updatedfreq || $deletedfreq) {

                activityLog(7, $package->_id, 2);

                return $this->packageRepository->transactionResponse(1, null, __tr('Package updated.'));
            }

            return $this->packageRepository->transactionResponse(14, null, 'Package not updated');
        });

        return $this->engineReaction($transactionResponse);
    }

    /**
     * Package prepare details data
     *
     * @param mix $packageIdOrUid
     *
     * @return  array
     *---------------------------------------------------------------- */

    public function preparePackageDetails($packageIdOrUid)
    {
        $package = $this->packageRepository->fetch($packageIdOrUid);

        if (__isEmpty($package)) {
            return $this->engineReaction(18, null, __tr('Package not found.'));
        }

        $allPackageItems = $this->packageRepository->getAllPackageItems($package->_id);
        $packageItems = $allPackageItems->pluck('items__id')->toArray();
        $items = $this->itemRepository->getAllItems();

        $services = [];
        $products = [];
        if (!__isEmpty($items)) {

            foreach ($items as $key => $item) {

                if (in_array($item->_id, $packageItems)) {
                    if ($item->type == 1) {
                        $services[] = [
                            'item_id' => $item->_id,
                            'title' => $item->title
                        ];
                    } else if ($item->type == 2) {
                        $products[] = [
                            'item_id' => $item->_id,
                            'title' => $item->title
                        ];
                    }
                }
            }
        }

        $frequencies = configItem('packages.frequencies');

        $addedFreq = [];
        if (!__isEmpty($package->packagePrices)) {
            foreach ($package->packagePrices as $key => $price) {
                $addedFreq[] = [
                    'package_price' => $price->price,
                    'formatted_price' => moneyFormat($price->price, true, false),
                    'frequency_title' => configItemString($frequencies, $price->frequency)
                ];
            }
        }

        $packageData = [
            '_id' => $package->_id,
            '_uid' => $package->_uid,
            'short_description' => $package->short_description,
            'title' => $package->title,
            'status' => techItemString($package->status),
            'package_frequency' => $addedFreq
        ];

        return $this->engineReaction(1, [
            'packageData' => $packageData,
            'packageitems' => [
                'services' => $services,
                'products' => $products
            ],
            'added_frequencies' => $addedFreq,
        ]);
    }

    /**
     * Package prepare details data
     *
     * @param mix $packageIdOrUid
     *
     * @return  array
     *---------------------------------------------------------------- */

    public function getAssignPackageData()
    {
        $allPackages = $this->packageRepository->fetchAllPackages();

        $packageData = [];
        if (!__isEmpty($allPackages)) {

            $frequencies = configItem('packages.frequencies');

            foreach ($allPackages as $key => $package) {

                $packageItems = [];
                if (!__isEmpty($package->packageItems)) {

                    foreach ($package->packageItems as $key => $packageItem) {

                        $packageItemsSlots = [];
                        if ($packageItem->item->status == 1) {

                            if (!__isEmpty($packageItem->item->itemSlots)) {

                                foreach ($packageItem->item->itemSlots as $key => $itemslot) {

                                    $fromtime = formatDateTime($itemslot->slot_time, 'g:i A');
                                    $endtime = addMinutesToTime($itemslot->slot_time, $itemslot->slot_duration_minutes);
                                    $packageItemsSlots[] = [
                                        'item_slot_id' => $itemslot->_id,
                                        'item_slot_uid' => $itemslot->_uid,
                                        'item_slot_title' => $itemslot->title,
                                        'item_slot_time' => $fromtime . ' - ' . $endtime,
                                        'item_slot_duration' => $itemslot->slot_duration_minutes,
                                        'item_slot_frequency' => $itemslot->slot_frequency,
                                    ];

                                }
                            }

                            $packageItems[] = [
                                'item_id' => $packageItem->items__id,
                                'package_item_id' => $packageItem->item->_id,
                                'package_item_uid' => $packageItem->item->_uid,
                                'package_item_title' => $packageItem->item->title,
                                'package_item_type' => $packageItem->item->type,
                                'package_item_slot_type' => $packageItem->item->slot_type,
                                'package_item_slots' => $packageItemsSlots
                            ];
                        }

                    }
                }

                $packagePrices = [];
                //check if not empty
                if (!__isEmpty($package->packagePrices)) {

                    foreach ($package->packagePrices as $key => $prices) {

                        $packagePrices[] = [
                            'frequency_id' => $prices->frequency,
                            'frequency_title' => configItemString($frequencies, $prices->frequency),
                            'price' => handleCurrencyAmount($prices->price),
                            'formatted_price' => moneyFormat($prices->price, true, false),
                            'uniqe_id' => uniqid()
                        ];
                    }
                }

                $packageData[] = [
                    'package_id' => $package->_id,
                    'package_uid' => $package->_uid,
                    'title' => $package->title,
                    'package_frequency' => $package->frequency,
                    'package_items' => $packageItems,
                    'package_prices' => $packagePrices
                ];
            }
        }

        return $this->engineReaction(1, [
            'assign_package_data' => $packageData,
            'currency_symbol' => getCurrencySymbol()
        ]);
    }

    /**
     * Package assign to user
     *
     * @param string $userId
     *
     * @return  array
     *---------------------------------------------------------------- */

    public function processPackageAssignToUser($inputData, $userId)
    {
        $user = $this->userRepository->fetch($userId);

        if (__isEmpty($user)) {
            return $this->engineReaction(18, null, __tr('User not found.'));
        }

        //check if package frequency selected
        if (__isEmpty($inputData['packageDetails']['selected_package_frequency'])) {
            return $this->engineReaction(2, null, __tr('Please select package frequency.'));
        }

        $userAuthority = $this->userRepository->fetchUserAuthorities($user->_id);

        /*---------------------- for assign package------------------------------------*/

        $transactionResponse = $this->packageRepository->processTransaction(function () use ($inputData, $user, $userAuthority) {

            if (!isset($inputData['packageDetails']['package_uid'])) {
                return $this->packageRepository->transactionResponse(2, null, 'Please Select Package.');
            }

            $package = $this->packageRepository->fetch($inputData['packageDetails']['package_uid']);

            if (__isEmpty($package)) {
                return $this->packageRepository->transactionResponse(18, null, 'Package not found.');
            }

            $assignPackageData = [
                'packages__id' => $package->_id,
                'price' => $inputData['packageDetails']['package_amount_paid'],
                'frequency' => $inputData['packageDetails']['selected_package_frequency'],
                'user_authorities__id' => $userAuthority->_id,
                'starts_at' => $inputData['packageDetails']['starts_at'],
                //'expiry_at'				=> calcFrequencyEndDate($inputData['packageDetails']['starts_at'], $inputData['packageDetails']['selected_package_frequency']),
            ];

            $packageitems = $inputData['packageDetails']['package_items'];

            $storedUserPackage = $this->packageRepository->storeUserPackage($assignPackageData);

            if ($storedUserPackage) {

                activityLog(7, $storedUserPackage->_id, 1);

                $userpackageSlots = [];
                if (!__isEmpty($packageitems)) {
                    foreach ($packageitems as $key => $packageitem) {
                        if ($packageitem['package_item_type'] == 1) {

                            foreach ($packageitem['package_item_slots'] as $key => $slot) {
                                if ($slot['item_slot_id'] == $packageitem['selected_slot']) {
                                    $userpackageSlots[] = [
                                        'status' => 1,
                                        'slot_time' => $slot['item_slot_time'],
                                        'slot_duration_minutes' => $slot['item_slot_duration'],
                                        'slot_frequency' => $slot['item_slot_frequency'],
                                        'user_packages__id' => $storedUserPackage->_id,
                                        'items__id' => $packageitem['item_id'],
                                        'slots__id' => $slot['item_slot_id'],
                                    ];
                                }
                            }

                        }
                    }
                }

                if (!__isEmpty($userpackageSlots)) {
                    $this->slotsRepository->storeUserPackageSlots($userpackageSlots);
                }

                return $this->packageRepository->transactionResponse(1, [
                    'user_package_uid' => $storedUserPackage->_uid,
                    'assign_package_data' => $this->prepareUserAssignedPackage($storedUserPackage->_uid)
                ], 'Package assigned to user.');
            }

            return $this->packageRepository->transactionResponse(2, null, 'Package not assigned.');

        });


        /*---------------------- for assign package------------------------------------*/

        return $this->engineReaction($transactionResponse);
    }

    /**
     * Package assign to member
     *
     *
     * @return  array
     *---------------------------------------------------------------- */

    public function processPackageAssignToMember($inputData)
    {

        if (empty($inputData['userData']['member_id'])) {
            return $this->engineReaction(18, null, __tr('Member not found.'));
        }

        //check if package frequency selected
        if (__isEmpty($inputData['packageDetails']['selected_package_frequency'])) {
            return $this->engineReaction(2, null, __tr('Please select package frequency.'));
        }

        /*---------------------- for assign package------------------------------------*/
        $transactionResponse = $this->packageRepository->processTransaction(function () use ($inputData) {

            if (!isset($inputData['packageDetails']['package_uid'])) {
                return $this->packageRepository->transactionResponse(2, null, 'Please Select Package.');
            }

            $package = $this->packageRepository->fetch($inputData['packageDetails']['package_uid']);

            if (__isEmpty($package)) {
                return $this->packageRepository->transactionResponse(18, null, 'Package not found.');
            }

            $assignPackageData = [
                'packages__id' => $package->_id,
                'price' => $inputData['packageDetails']['package_amount_paid'],
                'frequency' => $inputData['packageDetails']['selected_package_frequency'],
                'user_authorities__id' => 1,
                'starts_at' => $inputData['packageDetails']['starts_at'],
                "client_id" => $package->client_id,
                "member_id" => $inputData['userData']['member_id'],
                "memberName" => $inputData['userData']['name'],
                "memberPhone" => $inputData['userData']['phone'],
                'expiry_at' => calcFrequencyEndDate($inputData['packageDetails']['starts_at'], $inputData['packageDetails']['selected_package_frequency']),
            ];

            $packageitems = $inputData['packageDetails']['package_items'];

            $storedUserPackage = $this->packageRepository->storeUserPackage($assignPackageData);

            if ($storedUserPackage) {

                activityLog(7, $storedUserPackage->_id, 1);

                $userpackageSlots = [];
                if (!__isEmpty($packageitems)) {
                    foreach ($packageitems as $key => $packageitem) {
                        if ($packageitem['package_item_type'] == 1) {

                            foreach ($packageitem['package_item_slots'] as $key => $slot) {
                                if ($slot['item_slot_id'] == $packageitem['selected_slot']) {
                                    $userpackageSlots[] = [
                                        'status' => 1,
                                        'slot_time' => $slot['item_slot_time'],
                                        "client_id" => $package->client_id,
                                        "member_id" => $inputData['userData']['member_id'],
                                        'slot_duration_minutes' => $slot['item_slot_duration'],
                                        'slot_frequency' => $slot['item_slot_frequency'],
                                        'user_packages__id' => $storedUserPackage->_id,
                                        'items__id' => $packageitem['item_id'],
                                        'slots__id' => $slot['item_slot_id'],
                                    ];
                                }
                            }

                        }
                    }
                }

                if (!__isEmpty($userpackageSlots)) {
                    $this->slotsRepository->storeUserPackageSlots($userpackageSlots);
                }

                return $this->packageRepository->transactionResponse(1, [
                    'userData' => $inputData['userData'],
                    'user_package_uid' => $storedUserPackage->_uid,
                    'assign_package_data' => $this->prepareUserAssignedPackage($storedUserPackage->_uid)
                ], 'Package assigned to user.');
            }

            return $this->packageRepository->transactionResponse(2, null, 'Package not assigned.');

        });


        /*---------------------- for assign package------------------------------------*/

        return $this->engineReaction($transactionResponse);
    }

    /**
     * Fetch Active Users via athority Ids
     *
     * @return  array
     *---------------------------------------------------------------- */

    private function getUsers($userAuthorityIds = [])
    {
        return $this->userRepository->fetchMemberViaAuthorityIds($userAuthorityIds);

//        $usersData = [];
//
//        if (!__isEmpty($users)) {
//
//            foreach ($users as $user) {
////                if (!__isEmpty($user->profile) && isset($user->profile->profile_picture)) {
////                    $profilePicture = $user->profile->profile_picture;
////                } else {
////                    $profilePicture = '';
////                }
////                $profileMedia = mediaStorage('user_photo', ['{_uid}' => $user->_uid]) . '/' . $profilePicture;
////                $profilePicture = file_exists($profileMedia)
////                    ? mediaUrl('user_photo', ['{_uid}' => $user->_uid]) . '/' . $profilePicture
////                    : noUserThumbImageURL();
//                $usersData[] = [
//                    'member_id' => $user->_id,
//                    'name' => $user->name,
//                    'phone' => $user->username,
//                    'email' => $user->email,
////                    'mobile_number' => $user->mobile_number,
////                    'user_authority_id' => $user->user_authority_id,
////                    'created_at' => formatDateTime($user->created_at),
////                    'userRoleTitle' => $user->userRoleTitle,
////                    'status_id' => $user->status,
////                    'status' => configItem('status_codes', $user->status),
////                    'address_line_1' => isset($user->profile->address_line_1) ?
////                        $user->profile->address_line_1 : null,
////                    'address_line_2' => isset($user->profile->address_line_2) ?
////                        $user->profile->address_line_2 : null,
////                    'country' => isset($user->profile->country->name) ?
////                        $user->profile->country->name : null,
//                    'profile_picture' => noUserThumbImageURL()
//                ];
//
//            }
//        }
//
//        return $usersData;

    }

    /**
     * User Purchases
     *
     * @param mix $packageIdOrUid
     *
     * @return  array
     *---------------------------------------------------------------- */

    public function prepareUsersPurchases()
    {
        $userPackages = $this->packageRepository->fetchCustomerUserPackages()->toArray();
//        $userPackages = $this->packageRepository->fetchPackages()->toArray();

        $memberIds = array_values(array_unique(array_column($userPackages, 'member_id')));

        // get Customers
        $customers = $this->getUsers($memberIds)->toArray();

        $packagesData = [];

        if (!__isEmpty($userPackages)) {

            foreach ($userPackages as $userPackage) {
                if (!__isEmpty($userPackage['packages'])) {
                    foreach ($userPackage['packages'] as $package) {
                        $packagesData[$userPackage['member_id']][] = [
                            '_id' => $package['_id'],
                            '_uid' => $package['_uid'],
                            'title' => $package['title'],
                            'short_description' => $package['title'] . "\r\n" . $package['short_description'] ?? '',
                            'price' => $userPackage['price'],
                            'user_packages_id' => $userPackage['_id'],
                            'user_authorities__id' => $userPackage['user_authorities__id']
                        ];
                    }
                }
            }
        }

        return $this->engineReaction(1, [
            'purchases' => $packagesData,
            'users' => $customers
        ]);
    }


    /**
     * User Purchases
     *
     * @param mix $userPackageUid
     *
     * @return  array
     *---------------------------------------------------------------- */

    public function prepareUserPurchase($requestFor, $userPackageUid)
    {
        $userPurchase = $this->packageRepository->fetchUserPackageDetails($userPackageUid);

        if (__isEmpty($userPurchase)) {
            return $this->engineReaction(18, null, 'package not found');
        }

        $packageData = [];


        if (!__isEmpty($userPurchase->package)) {

            $package = $userPurchase->package;

            $packageData[] = [
                'rowId' => 'row1',
                'short_description' => $package->title . "\r\n" . $package['short_description'] ?? '',
                'amount' => (float)$userPurchase->price,
                'user_packages__id' => $userPurchase->_id
            ];
        }

        $frequency = $userPurchase->frequency;
        $customer = $userPurchase->customer;
        $dueDate = now();

        if ($frequency !== 1) { // Daily
            $dueDate = now();
        }

        $userPackageData = [
            'due_date' => formatDateTime($dueDate, 'Y-m-d'),
            'user_packages_id' => $userPurchase->_id,
            'items' => $packageData,
            'bill_period_from' => formatDateTime($userPurchase->starts_at, 'Y-m-d'),
            'bill_period_to' => formatDateTime(calcFrequencyEndDate($userPurchase->starts_at, $frequency), 'Y-m-d'),
            'customer_user_authorities__id' => 1,
            'short_description' => '',
            'total_bill_amount' => $userPurchase->price,
            'tax_amount' => 0,
            'discount_amount' => 0,
            'user' => 1
        ];

        $recentBill = $this->billRepository->fetchRecentPackageBill($userPurchase->_id);

        $recentBillData = [];

        if (!__isEmpty($recentBill)) {

            $recentBillData = [
                '_id' => $recentBill->_id,
                '_uid' => $recentBill->_uid,
                'billNo' => generateBillNo($recentBill->_uid),
                'description' => $recentBill->description,
                'frequency' => configItemString('packages.frequencies', $frequency),
                'amount' => moneyFormat($recentBill->amount, true, true),
                'discount_amount' => moneyFormat($recentBill->discount_amount, true, false),
                'tax_amount' => moneyFormat($recentBill->tax_amount, true, false),
                'bill_period_from' => formatDateTime($recentBill->bill_period_from, 'l jS F Y'),
                'bill_period_to' => formatDateTime($recentBill->bill_period_to, 'l jS F Y'),
                'due_date' => formatDateTime($recentBill->due_date, 'l jS F Y'),
                'status' => configItemString('bills.status', $recentBill->status)
            ];

            if ($requestFor === 'renew') {

                $userPackageData['bill_period_from'] = $recentBill->bill_period_to;
                $userPackageData['short_description'] = $recentBill->short_description;
                $userPackageData['bill_period_to'] = formatDateTime(calcFrequencyEndDate($recentBill->bill_period_to, $frequency), 'Y-m-d');
                $userPackageData['due_date'] = formatDateTime($recentBill->bill_period_to);
            }
        }

        return $this->engineReaction(1, [
            'user_package' => $userPackageData,
            'items' => $packageData,
            'recent_bill' => $recentBillData,
            'customer_info' => $customer
        ]);
    }

    /**
     * User Purchases
     *
     * @param mix $userPackageUid
     *
     * @return  array
     *---------------------------------------------------------------- */

    public function prepareUserAssignedPackage($userPackageUid)
    {
        $userPurchase = $this->packageRepository->fetchUserPackageDetails($userPackageUid);

        if (__isEmpty($userPurchase)) {
            return $this->engineReaction(18, null, 'package not found');
        }

        $packageData = [];

        if (!__isEmpty($userPurchase->package)) {

            $package = $userPurchase->package;

            $packageData = [
                'short_description' => $package->title,
                'assigned_on' => formatDateTime($package->created_at, 'l jS F Y'),
                'amount' => moneyFormat((float)$userPurchase->price, true, false),
                "frequency" => configItemString('packages.frequencies', $userPurchase->frequency),
                "starts_at" => formatDateTime($userPurchase->starts_at, 'l jS F Y'),
                'assigned_to' => $userPurchase->customer->name
            ];
        }

        return $packageData;
    }

    /**
     * get requirements details
     *
     * @return  array
     *---------------------------------------------------------------- */

    public function updateUserPackage($inputData)
    {
        $userpackage = $this->packageRepository->fetchUserPackageDetails($inputData['userPackageId']);

        if (__isEmpty($userpackage)) {
            return $this->engineReaction(18, null, __tr('User Package not found.'));
        }

        if (!isset($inputData['requestType'])) {
            return $this->engineReaction(2, null, __tr('Invalid Request.'));
        }

        if ($inputData['requestType'] == 1) {
            $updateData['price'] = $inputData['price'];
        } else {
            $updateData['expiry_at'] = $inputData['expiry_date'];
        }

        if ($this->packageRepository->updateUserPackage($userpackage, $updateData)) {

            return $this->engineReaction(1, null, __tr('User package updated.'));
        }

        return $this->engineReaction(14, null, __tr('User package not updated.'));
    }

    /**
     * get requirements details
     *
     * @return  array
     *---------------------------------------------------------------- */

    public function deleteUserSlot($inputData)
    {
        $userSlot = $this->slotsRepository->fetchUserSlot($inputData['itemSlotUid']);

        if (__isEmpty($userSlot)) {
            return $this->engineReaction(18, null, __tr('User package item not found.'));
        }

        if ($this->slotsRepository->deleteUserSlot($userSlot->_uid)) {

            return $this->engineReaction(1, null, __tr('User package item removed.'));
        }

        return $this->engineReaction(14, null, __tr('User package item not removed.'));
    }
}