<?php
/*
* PackageRepository.php - Repository file
*
* This file is part of the Package component.
*-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Package\Repositories;

use App\Yantrana\Base\BaseRepository;

use App\Yantrana\Components\Package\Models\PackageModel;
use App\Yantrana\Components\Package\Models\UserPackageModel;
use App\Yantrana\Components\Package\Models\PriceModel;
use App\Yantrana\Components\Package\Models\PackageItem;
use App\Yantrana\Components\Package\Interfaces\PackageRepositoryInterface;
use DB;

class PackageRepository extends BaseRepository
    implements PackageRepositoryInterface
{

    /**
     * Fetch the record of Package
     *
     * @param int || string $idOrUid
     *
     * @return    eloquent collection object
     *---------------------------------------------------------------- */

    public function fetch($idOrUid)
    {
        if (is_numeric($idOrUid)) {
            return PackageModel::with('packagePrices')->where('packages._id', $idOrUid)->first();
        }

        return PackageModel::with('packagePrices')->where('packages._uid', $idOrUid)->first();
    }

    /**
     * Fectch customer authorities ids
     *
     * @param int || string $idOrUid
     *
     * @return   eloquent collection object
     *---------------------------------------------------------------- */
    public function fetchCustomerUserPackages()
    {
        return UserPackageModel::with('packages')->whereHas('packages', function ($q) {
            $q->whereNull('expiry_at')
                ->orWhere(function ($que) {
                    $que->where('expiry_at', '>=', now());
                });
        })->get();
    }

    /**
     * Fectch customer authorities ids
     *
     * @param int || string $idOrUid
     *
     * @return   eloquent collection object
     *---------------------------------------------------------------- */
    public function fetchPackages($packagesIds = [])
    {
        if (!__isEmpty($packagesIds)) {

            return PackageModel::whereIn('_id', $packagesIds)->get();
        }

        return PackageModel::get();
    }

    /**
     * Fetch the record of Package
     *
     * @param int || string $idOrUid
     *
     * @return    eloquent collection object
     *---------------------------------------------------------------- */

    public function fetchPackageDetails($packageIdOrUid)
    {
        return PackageModel::where('packages._uid', $packageIdOrUid)
            ->with('packageItems')
            ->first();
    }

    /**
     * Fetch package datatable source
     *
     * @return  mixed
     *---------------------------------------------------------------- */

    public function fetchPackageDataTableSource()
    {
        $dataTableConfig = [
            'searchable' => [
                'title',
                'short_description'
            ],
            'fieldAlias' => [
                'formatted_price' => 'prices.price',
                'formatted_status' => 'packages.status'
            ]
        ];

        return PackageModel::dataTables($dataTableConfig)->toArray();
    }

    /**
     * Delete $package record and return response
     *
     * @param array $inputData
     *
     * @return  mixed
     *---------------------------------------------------------------- */

    public function deletePackage($package)
    {
        // Check if $package deleted
        if ($package->delete()) {

            return true;
        }

        return false;
    }

    /**
     * Store new package record and return response
     *
     * @param array $inputData
     *
     * @return  mixed
     *---------------------------------------------------------------- */

    public function storePackage($inputData)
    {
        $keyValues = [
            'title' => $inputData['title'],
            'short_description',
            'status'
        ];

        $newPackage = new PackageModel(['client_id' => $inputData['client_id']]);

        // Check if task testing record added then return positive response
        if ($newPackage->assignInputsAndSave($inputData, $keyValues)) {

            return $newPackage;

        }

        return false;
    }

    /**
     * Store new package record and return response
     *
     * @param array $inputData
     *
     * @return  mixed
     *---------------------------------------------------------------- */

    public function storePackagePrice($inputData, $packageId)
    {
        $keyValues = [
            'price' => $inputData['package_price'],
            'frequency' => $inputData['package_frequency'],
            'packages__id' => $packageId,
            'status' => 1
        ];

        $newPackagePrice = new PriceModel;

        // Check if task testing record added then return positive response
        if ($newPackagePrice->assignInputsAndSave($inputData, $keyValues)) {

            return $newPackagePrice;

        }

        return false;
    }


    /**
     * Store new package record and return response
     *
     * @param array $inputData
     *
     * @return  mixed
     *---------------------------------------------------------------- */

    public function storeFrequencyPrices($frequencies)
    {
        $price = new PriceModel;

        // Check if task testing record added then return positive response
        if ($price->prepareAndInsert($frequencies)) {

            return true;

        }

        return false;
    }

    /**
     * update package record and return response
     *
     * @param array $inputData
     *
     * @return  mixed
     *---------------------------------------------------------------- */

    public function updateFrequencyPrices($frequencies)
    {
        $price = new PriceModel;

        // Check if task testing record added then return positive response
        if ($price->batchUpdate($frequencies, '_id')) {

            return true;

        }

        return false;
    }

    /**
     * delete prices
     *
     * @param array $inputData
     *
     * @return  mixed
     *---------------------------------------------------------------- */

    public function deleteFrequencyPrices($priceIds, $packageId)
    {
        return PriceModel::where('prices.packages__id', $packageId)
            ->whereIn('prices._id', $priceIds)
            ->delete();
    }

    /**
     * Store new package record and return response
     *
     * @param array $inputData
     *
     * @return  mixed
     *---------------------------------------------------------------- */

    public function fetchPackagePrice($packageId)
    {
        return PriceModel::where('prices.packages__id', $packageId)->first();
    }

    /**
     * Store new package record and return response
     *
     * @param array $inputData
     *
     * @return  mixed
     *---------------------------------------------------------------- */

    public function storePackageItems($packageitems)
    {
        $items = new PackageItem;

        // Check if task testing record added then return positive response
        if ($items->prepareAndInsert($packageitems)) {

            return $items;

        }

        return false;
    }

    /**
     * Update package record and return response
     *
     * @param object $package
     * @param array $inputData
     *
     * @return  mixed
     *---------------------------------------------------------------- */

    public function update($modelObject, $inputData)
    {
        // Check if $package updated then return positive response
        if ($modelObject->modelUpdate($inputData)) {

            return true;

        }

        return false;
    }

    /**
     * get all package items by package
     *
     * @param int $packageId
     *
     * @return  mixed
     *---------------------------------------------------------------- */

    public function getAllPackageItems($packageId)
    {
        return PackageItem::where('package_items.packages__id', $packageId)->get();
    }

    /**
     * get all package-items by item
     *
     * @param int $itemId
     *
     * @return  mixed
     *---------------------------------------------------------------- */

    public function getAllPackageItemsByItem($itemId)
    {
        return PackageItem::where('package_items.items__id', $itemId)->get();
    }

    /**
     * delete package items
     *
     * @param array $packageItems
     * @param int $packageId
     *
     * @return  mixed
     *---------------------------------------------------------------- */

    public function deletePackageItems($packageItems, $packageId)
    {
        return PackageItem::where('package_items.packages__id', $packageId)
            ->whereIn('package_items.items__id', $packageItems)
            ->delete();
    }


    /**
     * fetch all packages
     *
     * @return  mixed
     *---------------------------------------------------------------- */

    public function fetchAllPackages()
    {
        return PackageModel::with('packageItems', 'packagePrices')
            ->where('packages.status', '=', 1)
            ->get();
    }

    /**
     * Store user assigned package
     *
     * @param array $inputData
     *
     * @return  mixed
     *---------------------------------------------------------------- */

    public function storeUserPackage($inputData)
    {
        $keyValues = [
            'status' => 1,
            'packages__id',
            'price',
            'starts_at',
            'user_authorities__id',
            'expiry_at',
            'frequency',
            "client_id",
            "member_id",
            "memberName",
            "memberPhone",
            "memberEmail",
        ];

        $userPackage = new UserPackageModel;

        // Check if task testing record added then return positive response
        if ($userPackage->assignInputsAndSave($inputData, $keyValues)) {

            return $userPackage;

        }

        return false;
    }

    /**
     * Fetch user purchases
     *
     * @param int || string $idOrUid
     *
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function fetchUsersPurchases($userPurcheUid = null)
    {
        return UserPackageModel::join('user_authorities', 'user_authorities._id', 'user_packages.user_authorities__id')
            ->join('users', 'users._id', 'user_authorities.users__id')
            ->join('packages', 'packages._id', '=', 'user_packages.packages__id')
            ->where('users.status', 1)
            ->join('package_items', function ($j) {
                $j->on('packages._id', '=', 'package_items.packages__id')
                    ->leftJoin('items', 'items._id', '=', 'package_items.items__id');
            })
            ->when(!__isEmpty($userPurcheUid), function ($q) use ($userPurcheUid) {
                $q->where('user_packages._uid', $userPurcheUid);
            })
            ->leftJoin('user_profiles', 'users._id', '=', 'user_profiles.users__id')
            ->leftJoin('countries', 'countries._id', '=', 'user_profiles.countries__id')
            ->select(
                __nestedKeyValues([
                    'packages' => ['title', 'short_description'],
                    'user_packages' => [
                        '_id', '_uid', 'packages__id',
                        'user_authorities__id', 'expiry_at', 'price', 'frequency'
                    ],
                    'users' => [
                        '_id as user_id',
                        '_uid as user_uid',
                        DB::raw('CONCAT(first_name, " ", last_name) AS name'),
                        'mobile_number'
                    ],
                    'package_items' => [
                        '_id as package_item_id',
                        '_uid as package_item_uid',
                        'items__id'
                    ],
                    'items' => [
                        'title as package_item_title'
                    ],
                    'user_profiles' => ['address_line_1', 'address_line_2'],
                    'countries' => ['name as country_name']
                ])
            )->get();
    }

    /**
     * Fetch user purchases
     *
     * @param int || string $idOrUid
     *
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function fetchUserPackageDetails($userPurcheUid)
    {
        return UserPackageModel::with(['package'])->where('_uid', $userPurcheUid)->first();
    }


    /**
     * fetch user assigned packages
     *
     * @return  mixed
     *---------------------------------------------------------------- */

    public function fetchMemberAssignedPackages($member_id)
    {
        return UserPackageModel::leftjoin('packages', 'user_packages.packages__id', '=', 'packages._id')
            ->with(['userSlots', 'billItems', 'packages' => function ($query) {
                $query->with('packageItems');
            }])
            ->where('user_packages.member_id', '=', $member_id)
            ->where('user_packages.packages__id', '!=', '')
            ->select(__nestedKeyValues([
                'packages' => [
                    '_id as package_id',
                    'title'
                ],
                'user_packages.*'
            ]))
            ->get();
    }

    /**
     * fetch all assigned packages
     *
     * @return  mixed
     *---------------------------------------------------------------- */

    public function getAllAssignedPackages($packageId)
    {
        return UserPackageModel::where('packages__id', '=', $packageId)->get();
    }

    /**
     * fetch user assigned packages
     *
     * @return  mixed
     *---------------------------------------------------------------- */

    public function fetchUserAssignedServices($userAuthorityId)
    {
        return UserPackageModel::with('userSlots')
            ->where('user_packages.user_authorities__id', '=', $userAuthorityId)
            ->where('user_packages.expiry_at', '>=', date('Y-m-d H:i:s'))
            ->get();
    }


    /**
     * fetch user assigned packages
     *
     * @return  mixed
     *---------------------------------------------------------------- */
    public function fetchPackagesCount()
    {
        return PackageModel::count();
    }

    /**
     * fetch user assigned packages
     *
     * @return  mixed
     *---------------------------------------------------------------- */
    public function fetchUserPurchasedCount()
    {
        return UserPackageModel::whereNotNull('packages__id')->count();
    }


    /**
     * fetch all packages
     *
     * @return  mixed
     *---------------------------------------------------------------- */

    public function getMyPackage($userAuthorityId)
    {
        return UserPackageModel::leftjoin('packages', 'user_packages.packages__id', '=', 'packages._id')
            ->with(['userAssignedSlot', 'packages' => function ($query) {
                $query->with('packageItems');
            }])
            ->where('user_packages.user_authorities__id', '=', $userAuthorityId)
            ->where('user_packages.packages__id', '!=', '')
            ->select(__nestedKeyValues([
                'packages' => [
                    '_id as package_id',
                    'title'
                ],
                'user_packages.*'
            ]))
            ->get();
    }

    /**
     * Update package record and return response
     *
     * @param object $package
     * @param array $inputData
     *
     * @return  mixed
     *---------------------------------------------------------------- */

    public function updateUserPackage($modelObject, $inputData)
    {
        // Check if $package updated then return positive response
        if ($modelObject->modelUpdate($inputData)) {

            return true;

        }

        return false;
    }

    /**
     * fetch user assigned packages
     *
     * @return  mixed
     *---------------------------------------------------------------- */

    // public function fetchUserPackagesWithTransactions($userAuthorityId)
    // {
    // 		return UserPackageModel::leftjoin('packages', 'user_packages.packages__id', '=', 'packages._id')
    // 			->with('bills', 'billItems')
    // 			->where('user_packages.user_authorities__id', '=', $userAuthorityId)
    // 			->where('user_packages.packages__id', '!=', '')
    // 			->select(__nestedKeyValues([
    //    					'packages' => [
    //    						'_id as package_id',
    //    						'title'
    //    					],
    //    					'user_packages.*'
    //    				]))
    // 			->get();
    // }
}