<?php
/*
* TransactionRepository.php - Repository file
*
* This file is part of the Transaction component.
*-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Transaction\Repositories;

use App\Yantrana\Base\BaseRepository;

use App\Yantrana\Components\Transaction\Models\TransactionModel;
use App\Yantrana\Components\Transaction\Interfaces\TransactionRepositoryInterface;
use DB;

class TransactionRepository extends BaseRepository
    implements TransactionRepositoryInterface
{

    /**
     * Fetch the record of Transaction
     *
     * @param int || string $idOrUid
     *
     * @return    eloquent collection object
     *---------------------------------------------------------------- */

    public function fetch($idOrUid)
    {
        if (is_numeric($idOrUid)) {

            return TransactionModel::where('_id', $idOrUid)->first();
        }

        return TransactionModel::where('_uid', $idOrUid)->first();
    }


    /**
     * Fetch transaction datatable source
     *
     * @return  mixed
     *---------------------------------------------------------------- */

    public function fetchAll()
    {
        return TransactionModel::leftJoin('bills', 'bills._id', '=', 'payment_transactions.bills__id')
            ->select(
                __nestedKeyValues([
                    'payment_transactions.*',
                    'bills' => ['_id as bill_id', '_uid as bill_uid'],
                ])
            )
            ->get();
    }

    /**
     * Fetch transaction datatable source
     *
     * @return  mixed
     *---------------------------------------------------------------- */

    public function fetchTransactionDataTableSource()
    {

        $dataTableConfig = [
            'searchable' => [
                'bill_uid' => 'bills._uid'
            ]
        ];

        return TransactionModel::leftJoin('bills', 'bills._id', '=', 'payment_transactions.bills__id')
            ->select(
                __nestedKeyValues([
                    'payment_transactions.*',
                    'bills' => ['_id as bill_id', '_uid as bill_uid'],
                ])
            )
            ->dataTables($dataTableConfig)
            ->toArray();

    }

    /**
     * Fetch transaction datatable source
     *
     * @return  mixed
     *---------------------------------------------------------------- */

    public function fetchTransactionOfThisMonth()
    {
        return TransactionModel::where(DB::raw('DATE(created_at)'), '>=', now()->startOfMonth())->get();
    }


    /**
     * Store new transaction record and return response
     *
     * @param array $inputData
     *
     * @return  mixed
     *---------------------------------------------------------------- */

    public function storeTransaction($inputData)
    {
        $user = getUserAuthInfo();
        $keyValues = [
            'amount',
            'short_description',
            'type',
            'status' => 2,
            'payment_method',
            'subtype',
            'bills__id',
            'towards_user_authorities__id',
            'client_id' => $user['profile']['client_id'],
            'user_authorities__id' => $user['user_authority_id'],
            '__data',
            'txn_id' => uniqid()
        ];;
        $newTransaction = new TransactionModel;

        // Check if task testing record added then return positive response
        if ($newTransaction->assignInputsAndSave($inputData, $keyValues)) {

            return $newTransaction;
        }

        return false;
    }

    /**
     * Update transaction record and return response
     *
     * @param object $transaction
     * @param array $inputData
     *
     * @return  mixed
     *---------------------------------------------------------------- */

    public function updateTransaction($transaction, $inputData)
    {
        // Check if transaction updated then return positive response
        if ($transaction->modelUpdate($inputData)) {

            return true;
        }

        return false;
    }


    /**
     * Fetch the record of Transaction
     *
     * @param int || string $idOrUid
     *
     * @return  eloquent collection object
     *---------------------------------------------------------------- */

    public function fetchBillTransactions($billId)
    {
        return TransactionModel::where([
            'bills__id' => $billId,
            'status' => 2
        ])->get();
    }


}