<?php
/*
* TransactionEngine.php - Main component file
*
* This file is part of the Transaction component.
*-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Transaction;

use App\Yantrana\Base\BaseEngine;

use App\Yantrana\Components\Transaction\Repositories\TransactionRepository;
use App\Yantrana\Components\Transaction\Interfaces\TransactionEngineInterface;
use App\Yantrana\Components\Bill\Repositories\BillRepository;
use App\Yantrana\Components\User\Repositories\UserRepository;

class TransactionEngine extends BaseEngine implements TransactionEngineInterface
{

    /**
     * @var  TransactionRepository $transactionRepository - Transaction Repository
     */
    protected $transactionRepository;

    /**
     * @var  BillRepository $billRepository - Bill Repository
     */
    protected $billRepository;

    /**
     * @var UserRepository - User Repository
     */
    protected $userRepository;


    /**
     * Constructor
     *
     * @param TransactionRepository $transactionRepository - Transaction Repository
     * @param UserRepository $userRepository - User Repository
     *
     * @return  void
     *-----------------------------------------------------------------------*/

    function __construct(
        TransactionRepository $transactionRepository,
        BillRepository $billRepository,
        UserRepository $userRepository)
    {
        $this->transactionRepository = $transactionRepository;
        $this->billRepository = $billRepository;
        $this->userRepository = $userRepository;
    }


    /**
     * Transaction datatable source
     *
     * @return array
     *---------------------------------------------------------------- */
    public function prepareTransactionList()
    {
        $transactionCollection = $this->transactionRepository->fetchTransactionDataTableSource();

        $config = configItem('transaction');

        $requireColumns = [
            '_id',
            '_uid',
            'type',
            'status',
            'subtype',
            'txn_id',
            'amount' => function ($key) {
                return moneyFormat($key['amount'], true, false);
            },
            'created_at' => function ($key) {
                return formatDateTime($key['created_at']);
            },
            'updated_at' => function ($key) {
                return formatDateTime($key['updated_at']);
            },
            'short_description',
            'formatted_type' => function ($key) use ($config) {
                return configItemString($config['types'], $key['type']);
            },
            'formatted_status' => function ($key) use ($config) {
                return configItemString($config['status'], $key['status']);
            },
            'formatted_subtype' => function ($key) use ($config) {
                return configItemString($config['subtypes'], $key['subtype']);
            },
            'bill_uid' => function ($key) {
                return generateBillNo($key['bill_uid']);
            },
        ];

        return $this->dataTableResponse($transactionCollection, $requireColumns);
    }


    /**
     * Transaction Add Support Data
     *
     * @return  array
     *---------------------------------------------------------------- */

    public function prepareTransactionSupportData()
    {
        $users = $this->userRepository->fetchMembers(); // Fetch user

        return $this->engineReaction(1, [
            'transactionsConfig' => configItem('transaction'),
            'users' => $users->toArray()
        ]);
    }

    /**
     * Get Bill Payment data
     *
     * @return  array
     *---------------------------------------------------------------- */
    public function preparBillPaymentInfo($billId, $billAmount)
    {
        $transactions = $this->transactionRepository->fetchBillTransactions($billId);

        $creditAmount = $transactions->where('type', 1)->sum('amount'); // payment
        $debitAmount = $transactions->where('type', 2)->sum('amount'); // refund
        $paidAmount = ($creditAmount - $debitAmount);

        return [
            'balance_amount' => ($billAmount - $paidAmount),
            'paid_amount' => $paidAmount,
            'bill_amount' => $billAmount,
            'bill_payment_paid' => ($paidAmount === $billAmount) ? true : false
        ];
    }

    /**
     * Update Bill Status
     *
     * @return  array
     *---------------------------------------------------------------- */
    private function updateBillStatus($id, $trnType = 1, $inputAmount = 0)
    {
        $trnType = (int)$trnType;

        $bill = $this->billRepository->fetch($id);

        $trnPayment = $this->preparBillPaymentInfo($bill->_id, $bill->amount);

        $status = ($trnPayment['balance_amount'] < 1) ? 3 : 2;

        if ($trnType === 2) { // Debit

            $status = ($inputAmount == $bill->amount) ? 4 : 2;
        }

        if ($this->billRepository->update($bill, ['status' => $status])) {
            activityLog(9, $bill->_id, 2);
        }

        return [
            'bill_no' => generateBillNo($bill->_uid),
            'paid_amount' => moneyFormat($trnPayment['paid_amount'], true, false),
            'bill_amount' => moneyFormat($trnPayment['bill_amount'], true, false),
            'balance_amount' => moneyFormat($trnPayment['balance_amount'], true, false),
            'bill_status' => configItemString(configItem('bills.status'), $bill->status)
        ];
    }

    /**
     * Check is overried bill amount
     *
     * @return  array
     *---------------------------------------------------------------- */

    private function verifyBillConditions($id, $inputAmount)
    {
        $bill = $this->billRepository->fetch($id);

        if (__isEmpty($bill)) {

            return [
                'invalid' => true,
                'message' => 'The bill does not exists.'
            ];
        }

        return [
            'invalid' => false
        ];
    }

    /**
     * Check is overried bill amount
     *
     * @return  array
     *---------------------------------------------------------------- */

    private function checkBillTrnBasedOnType($trnType, $bill, $inputAmount)
    {
        // Get payment info of bill
        $trnPayment = $this->preparBillPaymentInfo($bill->_id, $bill->amount);

        if ($inputAmount > $bill->amount) {

            $text = ($trnType === 1) ? 'pay' : 'refund';

            return [
                'invalid' => true,
                'message' => "Don't $text more than bill amount."
            ];
        }

        if ($trnType === 1) { // Credit

            if ($inputAmount > $trnPayment['balance_amount']) {

                return [
                    'invalid' => true,
                    'message' => __tr("Don't pay more than balance amount.")
                ];
            }
        }

        if ($trnType === 2) { // Debit

            if ($inputAmount > $trnPayment['paid_amount']) {

                return [
                    'invalid' => true,
                    'message' => __tr('You trying to refund __inputAmount__, Which is more that of paid:__paidAmount__ amount.', [
                        '__inputAmount__' => moneyFormat($inputAmount, true, false),
                        '__paidAmount__' => moneyFormat($trnPayment['paid_amount'], true, false)
                    ])
                ];
            }
        }

        return [
            'invalid' => false
        ];
    }

    /**
     * Transaction Add Support Data
     *
     * @return  array
     *---------------------------------------------------------------- */

    public function prepareUserBills($userId)
    {
        $users = $this->userRepository->fetchUser($userId);

        $bills = $this->billRepository->fetchUserBills($users->member_id, [1, 2, 3]);

        $billData = [];

        if (!__isEmpty($bills)) {

            $config = configItem('bills.status');

            foreach ($bills as $key => $bill) {
                $creditAmount = $bill->transactions->where('type', 1)->sum('amount'); // payment
                $debitAmount = $bill->transactions->where('type', 2)->sum('amount'); // refund
                $paidAmount = ($creditAmount - $debitAmount);
                $balanceAmount = ($bill->amount - $paidAmount);

                $statusTitle = configItemString($config, $bill->status);

                $billData[] = [
                    '_id' => $bill->_id,
                    '_uid' => $bill->_uid,
                    'bill_no' => generateBillNo($bill->_uid) . " - ( $statusTitle )",
                    'formated_bill_amount' => moneyFormat($bill->amount, true, false),
                    'formated_balance_amount' => moneyFormat($balanceAmount, true, false),
                    'formated_paid_amount' => moneyFormat($paidAmount, true, false),
                    'balance_amount' => $balanceAmount,
                    'paid_amount' => $paidAmount
                ];
            }
        }

        return $this->engineReaction(1, [
            'bills' => $billData
        ]);
    }

    /**
     * Transaction create
     *
     * @param array $inputData
     *
     * @return  array
     *---------------------------------------------------------------- */

    public function processTransactionCreate($inputData)
    {
        $bill = $this->billRepository->fetch($inputData['bills__id']);

        if (__isEmpty($bill)) {

            return [
                'invalid' => true,
                'message' => 'The bill does not exists.'
            ];
        }

        $inputData['status'] = 1;

        // if  payment -> credit , refund -> debit
        if ($inputData['subtype'] == 1) {
            $inputData['type'] = 1;
        } else if ($inputData['subtype'] == 2) {
            $inputData['type'] = 2;
        }

        $result = $this->checkBillTrnBasedOnType($inputData['type'], $bill, $inputData['amount']);

        // Check bill information is valid or not
        if ($result['invalid']) {
            return $this->engineReaction(2, null, $result['message']);
        }

        if ($transaction = $this->transactionRepository->storeTransaction($inputData)) {

            activityLog(11, $transaction->_id, 12);

            // Update bill status
            $this->updateBillStatus($transaction->bills__id, $inputData['type'], $inputData['amount']);

            return $this->engineReaction(1, null, __tr('Transaction added.'));
        }

        return $this->engineReaction(2, null, __tr('Transaction not added.'));
    }

    /**
     * Transaction process update
     *
     * @param mix $transactionIdOrUid
     * @param array $inputData
     *
     * @return  array
     *---------------------------------------------------------------- */

    public function processPayBill($billUid, $inputData)
    {
        $bill = $this->billRepository->fetch($billUid);

        if (__isEmpty($bill)) {

            return $this->engineReaction(18, null, "Bill not found");
        }

        $response = $this->transactionRepository->processTransaction(function ()
        use ($bill, $inputData) {

            $paymentData = [
                'amount' => $inputData['paying_amount'],
                'type' => 1,
                'status' => 2,
                'subtype' => 1,
                'bills__id' => $bill->_id,
                'short_description' => $inputData['short_description'] ?? '',
//                'towards_user_authorities__id' => $bill->customer_user_authorities__id,
                'payment_method' => $inputData['payment_method'],
                '__data' => [],
            ];

            if (!$transaction = $this->transactionRepository->storeTransaction($paymentData)) {

                return $this->billRepository->transactionResponse(2, null, __tr('Bill transaction not stored.'));
            }

            activityLog(11, $transaction->_id, 1);

            return $this->billRepository->transactionResponse(1);

        });

        // Check When Payment in process
        if ($response['0'] === 1) {

            return $this->engineReaction(1, [
                'transaction' => $this->updateBillStatus($bill->_id, 1)
            ], 'Payment sucessfully..!!');
        }

        return $this->engineReaction($response);
    }
}